# $Id: LispList.py,v 1.1 2017/02/08 21:57:33 leavens Exp $
import abc
class LispList(abc.ABC):
    pass
class Nil(LispList):
    def __init__(self):
        """Initialize this empty list"""
        pass
    def __eq__(self, lst):
        """Return True just when lst is also an instance of Nil."""
        return isinstance(lst, Nil)
    def __repr__(self):
        """Return a string representing this Nil instance."""
        return "Nil()"
    def __str__(self):
        """Return a string showing the elements of self."""
        return "[]"
    def isEmpty(self):
        """Return whether this list is empty."""
        return True

class Cons(LispList):
    def __init__(self, hd, tl):
        """Initialize this Cons with head hd and tail tl."""
        self.car = hd
        self.cdr = tl
    def __eq__(self, lst):
        """Return True just when self is structurally equivalent to lst."""
        return isinstance(lst, Cons) and lst.first() == self.first() \
            and lst.tail() == self.tail()
    def __repr__(self):
        """Return a string representing this list."""
        return "Cons(" + repr(self.first()) + ", " + repr(self.tail()) + ")"
    def elements_str(self):
        """Return a string of the elements of self, separated by commas."""
        if self.tail().isEmpty():
            return str(self.first())
        else:
            return str(self.first()) + ", " + self.tail().elements_str()
    def __str__(self):
        """Return a string showing the elements of self."""
        return "[" + self.elements_str() + "]"
    def isEmpty(self):
        """Return whether this list is empty."""
        return False
    def first(self):
        """Return the first element of this list."""
        return self.car
    def tail(self):
        """Return the rest of this list."""
        return self.cdr
